package com.company.mystorage;

import io.jmix.core.FileRef;
import io.jmix.core.FileStorageException;
import io.jmix.core.TimeSource;
import io.jmix.localfs.LocalFileStorage;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.springframework.lang.NonNull;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.time.ZonedDateTime;
import java.util.Date;


import static java.nio.file.StandardOpenOption.CREATE;

public class MyFileStorage extends LocalFileStorage
{
	public MyFileStorage(String storageName, String storageDir) {
		this.storageName = storageName;
		this.storageDir = storageDir;
		this.timeSource = new TimeSource() {
			@Override @NonNull
			public Date currentTimestamp() {
				return new Date();
			}

			@Override
			public long currentTimeMillis() {
				return System.currentTimeMillis();
			}

			@Override @NonNull
			public ZonedDateTime now() {
				return ZonedDateTime.now();
			}

		};
	}

	@Override
	public long saveStream(FileRef fileRef, @NonNull InputStream inputStream) {
		long size;
		File file = new File(fileRef.getFileName());

		try (OutputStream outputStream = Files.newOutputStream(file.toPath(), CREATE)) {
			size = IOUtils.copyLarge(inputStream, outputStream);
			outputStream.flush();
		} catch (IOException e) {
			Path relativePath = createRelativeFilePath(fileRef.getFileName());
			Path[] roots = getStorageRoots();
			Path path = roots[0].resolve(relativePath);
			FileUtils.deleteQuietly(path.toFile());
			throw new FileStorageException(FileStorageException.Type.IO_EXCEPTION, path.toAbsolutePath().toString(), e);
		}

		return size;
	}

}
